/***************************************************************************************
 * @file     reb_main.c
 * @version  V1.0
 * @date     31. March 2016
 *
 * @note
 * VORAGO Technologies 
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies. 
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY 
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT. 
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES 
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL 
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
#include <stdio.h>
#include "va108xx.h"
#include "reb_log.h"
#include "reb_board.h"
#include "reb_adt75.h"
#include "driver_common.h"
#include "reb_max11619.h"
#include "reb_timer.h"
#include "gpio_va108xx.h"

volatile uint8_t bounce_cnt = 0; 



//*********************************************************************************************
//    Project intended to drive JTAG port on another VA108xx MCU 
//     Port pin use table:
//     PA0 - TCK   (output)
//		 PA1 - TMS    (output)
//     PA2 - TRST   (output)
//     PA3 - TDI    (output)
//     PA4 - TDO    (input)


//*********************************************************************************************

#define sTCK   0
#define sTMS   1
#define sTRST  2
#define sTDI   3 
#define sTDO	 4 



//   *******  Global variables set here  *******

//   ****** subroutine to cycle TCK 1k times
//   clock will be high upon entry and exit. 

void TCK_1k(void)  
{
volatile	    uint32_t  cnt,cnt2   ; 
			
		for(cnt=0; cnt < 1000; cnt ++)
		{
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
	           for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;}   
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT shou
						 for(cnt2=0; cnt2 < 10; cnt2 ++)  { __NOP() ;} 
		}
	    
	return ; 
}

//   **  short delay  

void tclk_delay(void)
{
uint32_t  i  ;  

for(i=1;i<70;i++ )     
	{
			__NOP() ;  
	}//  delay here 

} 
//*********************************************************************************************
//    Subroutine to setup port pins and RESET the JTAG controller  
//        setup PA0-3 as output and PA4 as an input
//         drive reset pin of JTAG low for two TCLK cycles 
//         Move state machine to pause parking position (1 more clock with TMS = 0 
//         Data changes on falling edge of TCK and is sampled on rising edge. 


int32_t Reset_JTAG()  
{
	uint32_t i   ;  
		VOR_GPIO->BANK[0].DIR |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI) ; 
		VOR_GPIO->BANK[0].DIR &= ~(1<<sTDO)  ;  
	
		VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK)|(1 << sTMS)|(1 << sTRST)|(1 << sTDI);  // set all outputs to high state
	
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTRST);  // set trst low 
		tclk_delay()  ; 
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);   // set tclk low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);    // set tclk high -  DUT should process RST active here
		tclk_delay() ;  
	   VOR_GPIO->BANK[0].DATAOUT |= (1<<sTRST)  ;  	//  set TRST high 
	   VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);   // set tclk low 
		VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);    // set TMS low 
	  tclk_delay()  ;
	  VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);     // set tclk high
	  tclk_delay()  ;
	  tclk_delay()  ;  //   add second delay just for debug purposes to show break
	return 0  ;  
}




uint32_t rev_bit_order (uint32_t num_bits, uint32_t in_data) 
{
volatile	  uint32_t out_data, temp1, mask   ; 
	   temp1 = 0  ;
	   mask = 1 ; 
	  out_data = 0   ;  
	  while( num_bits -- > 0 ) 
	  {
	  temp1 = (in_data & mask) ; 
		if(temp1 != 0)
			{
				 out_data |= 0x1  ;  //  set lsb to 1 
			} 
     out_data <<=  1 ;  //  shift left 
		 mask <<= 1   ;    //   shift mask 1 position left
	  }
	  out_data >>= 1   ;   //   correct for one too many shifts left.  
	
 
   return out_data   ;  	
	 
}


//*********************************************************************************************
//    Subroutine to shift out data to data register of JTAG
//    input characters:  bitcnt - number of bits to shift  (1 to 65) 
//                       TDI_value - value to be shifted  (note:  LSBit is shifted out first)  
//    output character:  TDO_data -  This is the information clocked in on the TDO line during the shift.  
//
//     variable explanation:  out3 is a three bit field that consists of  TMS, TRST, TDI. This gets put on port pins on falling edge of tck. 
//    
//
//
//
//


uint32_t SDR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out,out3  ;  
    TDO_data = 0  ;  	
		out3 = 0  ;  
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 1 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[0].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTDI); 
				
			}

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[0].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =  1 1 0 to get from point where data is shifted to park) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
		   tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			 tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here

		return TDO_data ;  
}  


//  ****  trying to figure out why return statements are being optimized out  

uint32_t SDR_X(uint32_t bitcnt, uint32_t TDI_value) 
{
volatile	 int32_t  TDO_data, TDO_bitpos, temp1  ; 
volatile		 int8_t  bit_out,out3  ;  
    TDO_data = 0  ;  	
		out3 = 0  ;  
	  TDO_bitpos = 1   ;  
    temp1 = TDI_value  ;  
	
//  ** Move JTAG state machine position  (TMS =  1 1 0 to get from park to point where data is shifted) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			tclk_delay()  ;	
			VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
      tclk_delay()  ;
// ** Start shifting data out here  
	while (bitcnt-- > 0) 
	{
		  bit_out = TDI_value & 0x0001    ;
			TDI_value = TDI_value >> 1  ;    //   shift TDI_value one position to get ready for next bit
			if(bit_out == 1) 
			{
			       VOR_GPIO->BANK[0].DATAOUT |= (1 << sTDI);
			}
			else
			{
				    VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTDI); 
				
			}

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
 
	    tclk_delay()  ; 
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			temp1 = (VOR_GPIO->BANK[0].DATAIN & (1<<sTDO))  ;  //  sample TDO on rising edge of TCLK 
			if(temp1 != 0)
			{
				TDO_data |= TDO_bitpos ;  //  set bit in TDO if pin sample = 1, otherwise leave as zero 
			}
		
			TDO_bitpos = (  TDO_bitpos << 1 ) ;   //  move bit position to left 
		  tclk_delay() ;			
	}
	
	//  ** Move JTAG state machine position  (TMS =  1 1 0 to get from point where data is shifted to park) 

		  VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
		   tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT |= (1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here
			 tclk_delay()  ;
	     VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTCK);  // set tclk low 
			 VOR_GPIO->BANK[0].DATAOUT &= ~(1 << sTMS);   
	    tclk_delay()  ;
	    VOR_GPIO->BANK[0].DATAOUT |= (1 << sTCK);  // set tclk high -  DUT should process RST active here

		return TDO_data ;  
}  

//  ***  end of try to optimize out 

uint32_t SIR_sub(uint32_t bitcnt, uint32_t TDI_value) 
{
  volatile		uint32_t  TDO = 0, temp2   ;  
	bitcnt --  ; 
	temp2 = TDI_value  ; 
  TDO = temp2 + bitcnt  ;  	
	
	return TDO  ;
} 

//****************************************************************
uint32_t rev_bit_order2 (uint32_t num_bits, uint32_t in_data) 
{
	   uint32_t out_data   ;   
//  uint32_t out_data, temp1, mask   ; 
//	   temp1 = 0  ;
//	   mask = 1 ; 
//	  out_data = 0   ;  
	out_data = num_bits + in_data  ;     
	
 
   return out_data   ;  	
  
}

uint32_t check() 
{
	uint32_t out ; 
	  out =  VOR_GPIO->BANK[0].DATAIN  ; 
		VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE |= CLK_ENABLE_PORTA  ; 
	return out  ; 
}

int main()
{
	uint8_t cur_temp = 0; 
	int32_t ret_code = 0;
	uint16_t adc_data = 0;
volatile	uint32_t TDO_string[12],i,k,bill,k2,crypt[10]  ;  
	
	/* Enable clock for all peripherals */ 
	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE = ( CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_SPIA | CLK_ENABLE_SPIB | CLK_ENABLE_SPIC |
																					 CLK_ENABLE_UARTA | CLK_ENABLE_UARTB | CLK_ENABLE_I2CA | CLK_ENABLE_I2CB | CLK_ENABLE_IRQSEL |
																					 CLK_ENABLE_IOMGR | CLK_ENABLE_UTILITY | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM ); 
	
	  bill = check() ; 
		VOR_GPIO->BANK[0].DATAOUT |= (1 << bill)  ;
	  crypt[1] = bill  ;  
	  TCK_1k()  ; 
    bill = SIR_sub(11,0x7788)  ;    
	   crypt[2] =  bill   ;   
	  k2 = rev_bit_order2( 5,0x05)  ;
	   crypt[3] = k2  ;  
	  k = rev_bit_order( 9,0x01053)  ;
	    crypt[4] = k   ;  

		
		bill = SDR_X(16,0x137f)  ;  
	
		Reset_JTAG()  	  ;  
		TDO_string[0] = SDR_sub(16,0x137F)  ;  
	  i = TDO_string[0]  ;  
	
	
		while(1)  ;    // hang here during code debug phase  
	
	
#ifdef GPIO_DRIVER
	VOR_DRIVER_GPIO *PA10 = &Driver_GPIOA_10; 
#endif 	
	
	
	/* Initialize Logging module */ 
	VOR_Log_Init(PORTA_UARTA, LOG_DBG); 
	VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"Welcome to Vorago Technologies REB1 VA108xx software! \n");
	if( VOR_SYSCONFIG->PROCID == PROC_ID_TORCH )
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10820 MCU ...\n");
	else
		VOR_Log(LOG_INFO, ANSI_COLOR_GREEN"VA10800 MCU ... \n");
	VOR_Log(LOG_INFO, "Restarted... \n");
	VOR_Log(LOG_INFO, "Reset Status: 0x%08x \n", VOR_SYSCONFIG->RST_STAT); 
	
	/* Configure PA[10]. This GPIO will continously blink led D2. */ 	
#ifdef GPIO_DRIVER
	PA10->Initialize(NULL);
	PA10->Control(VOR_GPIO_CONTROL_MASK, VOR_GPIO_CTRL_ENABLE); 
	PA10->Control(VOR_GPIO_CONTROL_DIR, VOR_GPIO_DIR_OUTPUT); 
	PA10->Write(1);
#else 
	VOR_GPIO->BANK[0].DIR |= (1 << PORTA_10_D2); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_10_D2);
	VOR_GPIO->BANK[0].DATAOUT &= ~(1 << PORTA_10_D2); 
#endif 	
	/* Configure PA[11]. Input to blink led D3. */ 
	VOR_GPIO->BANK[0].DIR &= ~(1 << PORTA_11_SW_USER); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_11_SW_USER);
	
	/* Configure PA[7]. This GPIO will turn ON led D3. */ 
	VOR_GPIO->BANK[0].DIR |= (1 << PORTA_7_D3); 
	VOR_GPIO->BANK[0].DATAMASK |= (1 << PORTA_7_D3);
	VOR_GPIO->BANK[0].DATAOUT |= (1 << PORTA_7_D3); 
	
	/* Intialize MSec and Sec Timers */
	ret_code = VOR_TIM0_MSecInitialize(100); 	
	if( ret_code )
		VOR_Log(LOG_ERROR, "MSec Timer initialization failed! \n");

	ret_code = VOR_TIM1_SecInitialize(1); 	
	if( ret_code )
		VOR_Log(LOG_ERROR, "Sec Timer initialization failed! \n");
	
	/* Initialize temperature sensor - ADT75 */ 
	ret_code = VOR_ADT75_Initialize(); 
	if( ret_code )
		VOR_Log(LOG_ERROR, "ADT75 initialization failed! \n");

	/* Initialize SPI interface to Maxim ADC - MAX11619 */ 
	ret_code = VOR_MAX11619_Initialize(); 
	if( ret_code )
		VOR_Log(LOG_ERROR, "MAX11619 initialization failed! \n");
	
	while(1) {
		if( VOR_TIM1_SecExpired() ) {
			/* Retrieve current temperature */ 
			ret_code = VOR_ADT75_GetTemp(&cur_temp);
			if( ret_code )
				VOR_Log(LOG_ERROR, "ADT75 read failed! \n");
			else
				VOR_Log(LOG_INFO, "\nTemperature: %d(C)\n", cur_temp); 
		
			/* Configure Maxim ADC */ 
			ret_code = VOR_MAX11619_Configure();
			if( ret_code )
				VOR_Log(LOG_ERROR, "MAX11619 configuration failed! \n");
		
			/* Retrieve sampled data from ADC */ 
			ret_code = VOR_MAX11619_GetData(&adc_data);	
			if( ret_code )
				VOR_Log(LOG_ERROR, "MAX11619 get data failed! \n"); 
			else
				VOR_Log(LOG_INFO, "\nADC O/P (0(0V) to 1023(3.3V)): %d\n", adc_data);  
			
			VOR_TIM1_SecClear(); 	
		}	
		if( VOR_TIM0_MSecExpired() ) {
#ifdef GPIO_DRIVER			
			PA10->Write(1); 
#else
			VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_10_D2);		
#endif 			
			if( !(VOR_GPIO->BANK[0].DATAIN & (1 << PORTA_11_SW_USER)) )
				bounce_cnt++; 
			
			if(bounce_cnt >= 2) {
				VOR_GPIO->BANK[0].DATAOUT ^= (1 << PORTA_7_D3); 
				bounce_cnt = 0; 
			}
			VOR_TIM0_MSecClear(); 
		}
	}
}
